@@ -39,6 +39,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include <totem-pl-parser.h>
+#include <totem-scrsaver.h>
 
 #include <dbus/dbus-glib.h>
 
@@ -53,7 +54,7 @@
 #include "totem-glow-button.h"
 #include "video-utils.h"
 
-#include "totem-plugin-viewer-commands.h"
+#include "totem-plugin-viewer-constants.h"
 #include "totem-plugin-viewer-options.h"
 #include "totempluginviewer-marshal.h"
 
@@ -69,19 +70,13 @@ GtkWidget *totem_pp_create (void);
 #define DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT 0
 #endif
 
-typedef enum {
-	STATE_PLAYING,
-	STATE_PAUSED,
-	STATE_STOPPED,
-	LAST_STATE
-} TotemStates;
-
 typedef enum {
 	TOTEM_PLUGIN_TYPE_BASIC,
 	TOTEM_PLUGIN_TYPE_GMP,
 	TOTEM_PLUGIN_TYPE_COMPLEX,
 	TOTEM_PLUGIN_TYPE_NARROWSPACE,
 	TOTEM_PLUGIN_TYPE_MULLY,
+	TOTEM_PLUGIN_TYPE_CONE,
 	TOTEM_PLUGIN_TYPE_LAST
 } TotemPluginType;
 
@@ -94,6 +89,12 @@ typedef enum {
 
 typedef GObjectClass TotemEmbeddedClass;
 
+typedef struct _TotemPlItem {
+	char *uri;
+	int duration;
+	int starttime;
+} TotemPlItem;
+
 typedef struct _TotemEmbedded {
 	GObject parent;
 
@@ -103,6 +104,7 @@ typedef struct _TotemEmbedded {
 	GtkWidget *pp_button;
 	GtkWidget *pp_fs_button;
 	TotemStatusbar *statusbar;
+	TotemScrsaver *scrsaver;
 	int width, height;
 	const char *mimetype;
 	char *base_uri;
@@ -114,7 +116,7 @@ typedef struct _TotemEmbedded {
 	TotemStates state;
 	GdkCursor *cursor;
 
-	/* Playlist */
+	/* Playlist, a GList of TotemPlItem */
 	GList *playlist, *current;
 	guint parser_id;
 	int num_items;
@@ -186,8 +188,10 @@ enum {
 	BUTTON_PRESS,
 	START_STREAM,
 	STOP_STREAM,
+	TICK,
 	LAST_SIGNAL
 };
+
 static int signals[LAST_SIGNAL] = { 0 };
 
 static void
@@ -212,12 +216,13 @@ totem_embedded_finalize (GObject *object)
 
 static void totem_embedded_class_init (TotemEmbeddedClass *klass)
 {
-	GType param_types[2];
+	GType param_types[3];
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = totem_embedded_finalize;
 
 	param_types[0] = param_types[1] = G_TYPE_UINT;
+	param_types[2] = G_TYPE_STRING;
 	signals[BUTTON_PRESS] =
 		g_signal_newv ("button-press",
 				G_TYPE_FROM_CLASS (object_class),
@@ -244,6 +249,14 @@ static void totem_embedded_class_init (TotemEmbeddedClass *klass)
 				g_cclosure_marshal_VOID__VOID,
 				G_TYPE_NONE,
 				0, NULL);
+	signals[TICK] =
+		g_signal_newv ("tick",
+				G_TYPE_FROM_CLASS (object_class),
+				G_SIGNAL_RUN_LAST,
+				NULL /* class closure */,
+				NULL /* accu */, NULL /* accu data */,
+				totempluginviewer_marshal_VOID__UINT_UINT_STRING,
+				G_TYPE_NONE, 3, param_types);
 }
 
 static void
@@ -262,6 +275,16 @@ totem_embedded_error_and_exit (char *title, char *reason, TotemEmbedded *emb)
 	totem_embedded_exit (emb);
 }
 
+static void
+totem_embedded_save_volume (TotemEmbedded *emb, double volume)
+{
+	GConfClient *gc;
+
+	gc = gconf_client_get_default ();
+	gconf_client_set_int (gc, GCONF_PREFIX"/volume", (int) (volume * 100.0), NULL);
+	g_object_unref (G_OBJECT (gc));
+}
+
 static void
 totem_embedded_set_error (TotemEmbedded *emb,
 			  char *primary,
@@ -276,22 +299,16 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 {
 	GtkWidget *image;
 	gchar *id;
-	static const char *states[] = {
-		"PLAYING",
-		"PAUSED",
-		"STOPPED",
-		"INVALID"
-	};
 
 	if (state == emb->state)
 		return;
 
-	g_message ("Viewer state: %s", states[state]);
+	g_message ("Viewer state: %s", totem_states[state]);
 
 	image = gtk_button_get_image (GTK_BUTTON (emb->pp_button));
 
 	switch (state) {
-	case STATE_STOPPED:
+	case TOTEM_STATE_STOPPED:
 		id = GTK_STOCK_MEDIA_PLAY;
 		totem_statusbar_set_text (emb->statusbar, _("Stopped"));
 		totem_statusbar_set_time_and_length (emb->statusbar, 0, 0);
@@ -303,11 +320,11 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 				 emb->cursor);
 		}
 		break;
-	case STATE_PAUSED:
+	case TOTEM_STATE_PAUSED:
 		id = GTK_STOCK_MEDIA_PLAY;
 		totem_statusbar_set_text (emb->statusbar, _("Paused"));
 		break;
-	case STATE_PLAYING:
+	case TOTEM_STATE_PLAYING:
 		id = GTK_STOCK_MEDIA_PAUSE;
 		totem_statusbar_set_text (emb->statusbar, _("Playing"));
 		if (emb->href_uri == NULL && emb->hidden == FALSE) {
@@ -321,6 +338,8 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 		break;
 	}
 
+	if (emb->scrsaver != NULL)
+		totem_scrsaver_set_state (emb->scrsaver, (state == TOTEM_STATE_PLAYING) ? FALSE : TRUE);
 	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
 	gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (emb->pp_fs_button), id);
 
@@ -334,6 +353,7 @@ totem_embedded_pad_pixbuf_for_size (GdkPixbuf *pixbuf,
 	GdkPixbuf *logo;
 	guchar *pixels;
 	int rowstride, i;
+	int dest_x, dest_y;
 
 	logo = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 			       TRUE, 8, width, height);
@@ -345,13 +365,15 @@ totem_embedded_pad_pixbuf_for_size (GdkPixbuf *pixbuf,
 		memset (pixels + i * rowstride, 0, width * 4);
 	}
 
+	dest_x = (width - gdk_pixbuf_get_width (pixbuf)) / 2;
+	dest_y = (height - gdk_pixbuf_get_height (pixbuf)) / 2;
 	gdk_pixbuf_copy_area (pixbuf,
 			      0, 0,
-			      gdk_pixbuf_get_width (pixbuf),
-			      gdk_pixbuf_get_height (pixbuf),
+			      MIN (gdk_pixbuf_get_width (logo), gdk_pixbuf_get_width (pixbuf)),
+			      MIN (gdk_pixbuf_get_height (logo), gdk_pixbuf_get_height (pixbuf)),
 			      logo,
-			      (width - gdk_pixbuf_get_width (pixbuf)) / 2,
-			      (height - gdk_pixbuf_get_height (pixbuf)) / 2);
+			      dest_x,
+			      dest_y);
 
 	return logo;
 }
@@ -364,7 +386,7 @@ totem_embedded_set_logo_by_name (TotemEmbedded *embedded,
 	GdkPixbuf *logo, *padded;
 	int size, width, height;
 
-	totem_embedded_set_state (embedded, STATE_STOPPED);
+	totem_embedded_set_state (embedded, TOTEM_STATE_STOPPED);
 
 	if (embedded->audioonly != FALSE)
 		return;
@@ -397,6 +419,7 @@ totem_embedded_set_pp_state (TotemEmbedded *emb, gboolean state)
 
 static gboolean
 totem_embedded_open_internal (TotemEmbedded *emb,
+			      gboolean start_play,
 			      GError **error)
 {
 	GError *err = NULL;
@@ -420,7 +443,7 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 		return FALSE;
 	}
 
-	g_message ("totem_embedded_open_internal '%s' is-browser-stream %d", uri, emb->is_browser_stream);
+	g_message ("totem_embedded_open_internal '%s' is-browser-stream %d start-play %d", uri, emb->is_browser_stream, start_play);
 
 	bacon_video_widget_set_logo_mode (emb->bvw, FALSE);
 
@@ -436,7 +459,7 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 		/* FIXME we haven't even started sending yet! */
 		//g_signal_emit (emb, signals[STOP_STREAM], 0);
 
-		totem_embedded_set_state (emb, STATE_STOPPED);
+		totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 		totem_embedded_set_logo_by_name (emb, "image-missing");
 
 		errint = g_error_new (TOTEM_EMBEDDED_ERROR_QUARK,
@@ -453,10 +476,11 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 
 		totem_embedded_set_pp_state (emb, FALSE);
 	} else {
-		if (emb->playlist == NULL && totem_embedded_play (emb, NULL) != FALSE) {
-			totem_embedded_set_state (emb, STATE_PLAYING);
-		}
-		totem_embedded_set_pp_state (emb, TRUE);
+		/* FIXME we shouldn't even do that here */
+		if (start_play)
+			totem_embedded_play (emb, NULL);
+		else
+			totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
 	}
 
 	totem_embedded_update_menu (emb);
@@ -474,8 +498,13 @@ totem_embedded_play (TotemEmbedded *emb,
 {
 	GError *err = NULL;
 
+	g_return_val_if_fail (emb->current_uri != NULL, FALSE);
+
+	totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), FALSE);
+
 	if (bacon_video_widget_play (emb->bvw, &err) != FALSE) {
-		totem_embedded_set_state (emb, STATE_PLAYING);
+		totem_embedded_set_state (emb, TOTEM_STATE_PLAYING);
+		totem_embedded_set_pp_state (emb, TRUE);
 	} else {
 		g_warning ("Error in bacon_video_widget_play: %s", err->message);
 		g_error_free (err);
@@ -488,8 +517,8 @@ static gboolean
 totem_embedded_pause (TotemEmbedded *emb,
 		      GError **error)
 {
+	totem_embedded_set_state (emb, TOTEM_STATE_PAUSED);
 	bacon_video_widget_pause (emb->bvw);
-	totem_embedded_set_state (emb, STATE_PAUSED);
 
 	return TRUE;
 }
@@ -498,8 +527,8 @@ static gboolean
 totem_embedded_stop (TotemEmbedded *emb,
 		     GError **error)
 {
+	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 	bacon_video_widget_stop (emb->bvw);
-	totem_embedded_set_state (emb, STATE_STOPPED);
 
 	return TRUE;
 }
@@ -564,6 +593,16 @@ totem_embedded_set_error_logo (TotemEmbedded *embedded,
 	return TRUE;
 }
 
+static gboolean
+totem_embedded_set_volume (TotemEmbedded *embedded,
+			   gdouble volume,
+			   GError *error)
+{
+	g_message ("totem_embedded_set_volume: %f", volume);
+	bacon_video_widget_set_volume (embedded->bvw, volume);
+	return TRUE;
+}
+
 /* Copied from nautilus-program-choosing.c */
 
 extern char **environ;
@@ -894,6 +933,8 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	g_free (old_uri);
 	g_free (old_base);
 	g_free (old_href);
+	g_free (emb->stream_uri);
+	emb->stream_uri = NULL;
 }
 
 static gboolean
@@ -908,7 +949,7 @@ totem_embedded_open_uri (TotemEmbedded *emb,
 
 	totem_embedded_set_uri (emb, uri, base_uri, FALSE);
 
-	return totem_embedded_open_internal (emb, error);
+	return totem_embedded_open_internal (emb, TRUE, error);
 }
 
 static gboolean
@@ -922,10 +963,13 @@ totem_embedded_open_stream (TotemEmbedded *emb,
 	bacon_video_widget_close (emb->bvw);
 
 	totem_embedded_set_uri (emb, uri, base_uri, TRUE);
+	/* We can only have one item in the "playlist" when
+	 * we open a browser stream */
+	emb->num_items = 1;
 
 	/* FIXME: consume any remaining input from stdin */
 
-	return totem_embedded_open_internal (emb, error);
+	return totem_embedded_open_internal (emb, TRUE, error);
 }
 
 static gboolean
@@ -945,6 +989,7 @@ static gboolean
 totem_embedded_open_playlist_item (TotemEmbedded *emb,
 				   GList *item)
 {
+	TotemPlItem *plitem;
 	gboolean eop;
 
 	if (!emb->playlist)
@@ -960,16 +1005,29 @@ totem_embedded_open_playlist_item (TotemEmbedded *emb,
 	emb->current = item;
 	g_assert (item != NULL);
 
+	plitem = (TotemPlItem *) item->data;
+
 	totem_embedded_set_uri (emb,
-				(const char *) item->data,
+				(const char *) plitem->uri,
 			        emb->base_uri /* FIXME? */,
 			        FALSE);
 
 	bacon_video_widget_close (emb->bvw);
-	if (totem_embedded_open_internal (emb, NULL /* FIXME */) &&
-	    ((eop != FALSE && emb->repeat != FALSE)
-		|| (eop == FALSE))) {
-		totem_embedded_play (emb, NULL);
+	if (totem_embedded_open_internal (emb, FALSE, NULL /* FIXME */)) {
+		if (plitem->starttime > 0) {
+			gboolean retval;
+
+			g_message ("Seeking to %d seconds for starttime", plitem->starttime);
+			retval = bacon_video_widget_seek_time (emb->bvw,
+							       plitem->starttime * 1000,
+							       NULL /* FIXME */);
+			if (!retval)
+				return TRUE;
+		}
+
+		if ((eop != FALSE && emb->repeat != FALSE) || (eop == FALSE)) {
+		    	    totem_embedded_play (emb, NULL);
+		}
 	}
 
 	return TRUE;
@@ -996,7 +1054,28 @@ totem_embedded_set_local_file (TotemEmbedded *emb,
 	totem_embedded_set_uri (emb, file_uri, base_uri, FALSE);
 	g_free (file_uri);
 
-	return totem_embedded_open_internal (emb, error);
+	return totem_embedded_open_internal (emb, TRUE, error);
+}
+
+static gboolean
+totem_embedded_set_local_cache (TotemEmbedded *emb,
+				const char *path,
+				GError **error)
+{
+	char *file_uri;
+
+	/* FIXME Should also handle playlists */
+	if (!emb->is_browser_stream)
+		return TRUE;
+
+	file_uri = g_filename_to_uri (path, NULL, error);
+	if (!file_uri)
+		return FALSE;
+
+	emb->stream_uri = emb->current_uri;
+	emb->current_uri = file_uri;
+
+	return TRUE;
 }
 
 static gboolean
@@ -1186,20 +1265,96 @@ on_preferences1_activate (GtkButton *button, TotemEmbedded *emb)
 static void
 on_play_pause (GtkWidget *widget, TotemEmbedded *emb)
 {
-	if (emb->state == STATE_PLAYING) {
+	if (emb->state == TOTEM_STATE_PLAYING) {
 		totem_embedded_pause (emb, NULL);
 	} else {
-		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), FALSE);
-
 		if (emb->current_uri == NULL) {
+			totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), FALSE);
 			g_signal_emit (emb, signals[BUTTON_PRESS], 0,
-				       GDK_CURRENT_TIME, 0);
+				       gtk_get_current_event_time (), 0);
 		} else {
 			totem_embedded_play (emb, NULL);
 		}
 	}
 }
 
+static void
+popup_menu_position_func (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *button) 
+{
+	GtkWidget *widget = GTK_WIDGET (button);
+	GtkRequisition menu_req;
+	GtkTextDirection direction;
+
+	gtk_widget_size_request (GTK_WIDGET (menu), &menu_req);
+
+	direction = gtk_widget_get_direction (widget);
+
+	gdk_window_get_origin (widget->window, x, y);
+	*x += widget->allocation.x;
+	*y += widget->allocation.y;
+
+	if (direction == GTK_TEXT_DIR_LTR)
+		*x += MAX (widget->allocation.width - menu_req.width, 0);
+	else if (menu_req.width > widget->allocation.width)
+		*x -= menu_req.width - widget->allocation.width;
+
+	/* This might not work properly if the popup button is right at the
+	 * top of the screen, but really, what are the chances */
+	*y -= menu_req.height;
+
+	*push_in = FALSE;
+}
+
+static void
+popup_menu_over_arrow (GtkToggleButton *button,
+		       GtkMenu *menu,
+		       GdkEventButton *event)
+{
+	gtk_menu_popup (menu, NULL, NULL,
+			(GtkMenuPositionFunc) popup_menu_position_func,
+			button,
+			event ? event->button : 0,
+			event ? event->time : gtk_get_current_event_time ());
+}
+
+static void
+on_popup_button_toggled (GtkToggleButton *button, TotemEmbedded *emb)
+{
+	GtkMenu *menu;
+
+	menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
+
+	if (gtk_toggle_button_get_active (button) && !GTK_WIDGET_VISIBLE (menu)) {
+		/* we get here only when the menu is activated by a key
+		 * press, so that we can select the first menu item */
+		popup_menu_over_arrow (button, menu, NULL);
+		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
+	}
+}
+
+static gboolean
+on_popup_button_button_pressed (GtkToggleButton *button,
+				GdkEventButton *event,
+				TotemEmbedded *emb)
+{
+	if (event->button == 1) {
+		GtkMenu *menu;
+
+		menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
+		if (!GTK_WIDGET_VISIBLE (menu)) {
+			popup_menu_over_arrow (button, menu, event);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+		} else {
+			gtk_menu_popdown (menu);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+		}
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
 on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 {
@@ -1210,11 +1365,15 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 
 	bacon_video_widget_close (emb->bvw);
 
-	if (emb->current_uri)
-		new_uri = totem_resolve_relative_link (emb->current_uri, mrl);
+	/* We are using a local cache, so we resolve against the stream_uri */
+	if (emb->stream_uri)
+		new_uri = totem_pl_parser_resolve_url (emb->stream_uri, mrl);
+	/* We don't have a local cache, so resolve against the URI */
+	else if (emb->current_uri)
+		new_uri = totem_pl_parser_resolve_url (emb->current_uri, mrl);
 	/* FIXME: not sure that this is actually correct... */
 	else if (emb->base_uri)
-		new_uri = totem_resolve_relative_link (emb->base_uri, mrl);
+		new_uri = totem_pl_parser_resolve_url (emb->base_uri, mrl);
 
 	if (!new_uri)
 		return;
@@ -1226,10 +1385,9 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 
 	totem_embedded_set_uri (emb, new_uri, emb->base_uri /* FIXME? */, FALSE);
 
-	totem_embedded_set_state (emb, STATE_STOPPED);
+	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 
-	if (totem_embedded_open_internal (emb, NULL /* FIXME? */) != FALSE)
-		totem_embedded_play (emb, NULL);
+	totem_embedded_open_internal (emb, TRUE, NULL /* FIXME? */);
 }
 
 static void
@@ -1250,6 +1408,17 @@ totem_embedded_toggle_fullscreen (TotemEmbedded *emb)
 		
 		gtk_action_set_sensitive (fs_action, TRUE);
 	} else {
+		GdkRectangle rect;
+		int monitor;
+
+		/* Move the fullscreen window to the screen where the
+		 * video widget currently is */
+		monitor = gdk_screen_get_monitor_at_window (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
+							    GTK_WIDGET (emb->bvw)->window);
+		gdk_screen_get_monitor_geometry (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
+						 monitor, &rect);
+		gtk_window_move (GTK_WINDOW (emb->fs_window), rect.x, rect.y);
+
 		gtk_widget_reparent (GTK_WIDGET (emb->bvw), emb->fs_window);
 		bacon_video_widget_set_fullscreen (emb->bvw, TRUE);
 		gtk_window_fullscreen (GTK_WINDOW (emb->fs_window));
@@ -1273,47 +1442,59 @@ on_video_button_press_event (BaconVideoWidget *bvw,
 			     TotemEmbedded *emb)
 {
 	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
-	gboolean handled = FALSE;
+	GtkMenu *menu;
 
 	//g_print ("button-press type %d button %d state %d play-state %d\n",
 	//	 event->type, event->button, state, emb->state);
 
-	if (event->type == GDK_BUTTON_PRESS &&
-	    event->button == 1 &&
-	    state == 0 &&
-	    emb->state == STATE_STOPPED) {
-		g_message ("emitting signal");
-		g_signal_emit (emb, signals[BUTTON_PRESS], 0,
-			       event->time,
-			       event->button);
-	} else if (event->type == GDK_BUTTON_PRESS &&
-		   event->button == 3 &&
-		   state == 0) {
-		GtkMenu *menu;
+	menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
+
+	if (event->type == GDK_BUTTON_PRESS && event->button == 1 && state == 0 && emb->state == TOTEM_STATE_STOPPED) {
+		if (!GTK_WIDGET_VISIBLE (menu)) {
+			g_message ("emitting signal");
+			g_signal_emit (emb, signals[BUTTON_PRESS], 0,
+				       event->time,
+				       event->button);
+		} else {
+			gtk_menu_popdown (menu);
+		}
+	} else if (event->type == GDK_BUTTON_PRESS && event->button == 3 && state == 0) {
+		GtkWidget *popup_button;
+
+		popup_button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "popup_button"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (popup_button), FALSE);
 
-		menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
 		gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
 				event->button, event->time);
-
-		handled = TRUE;
 	}
 
-	return handled;
+	return FALSE;
 }
 
 static void
-on_eos_event (GtkWidget *bvw, TotemEmbedded *emb)
+on_eos_event (BaconVideoWidget *bvw, TotemEmbedded *emb)
 {
-	totem_embedded_set_state (emb, STATE_STOPPED);
+	gboolean start_play;
+
+	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 	gtk_adjustment_set_value (emb->seekadj, 0);
 
 	/* FIXME: the plugin needs to handle EOS itself, e.g. for QTNext */
 
+	start_play = (emb->repeat != FALSE && emb->autostart);
+
 	/* No playlist if we have fd://0, right? */
 	if (emb->is_browser_stream) {
-		/* FIXME: totem_embedded_set_state missing? */
-		/* FIXME: should find a way to enable playback of the stream again without re-requesting it */
-		totem_embedded_set_pp_state (emb, FALSE);
+		/* Verify that we had a SetLocalCache */
+		if (g_str_has_prefix (emb->current_uri, "file://") != FALSE) {
+			emb->num_items = 1;
+			emb->is_browser_stream = FALSE;
+			bacon_video_widget_close (emb->bvw);
+			totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
+		} else {
+			/* FIXME: should find a way to enable playback of the stream again without re-requesting it */
+			totem_embedded_set_pp_state (emb, FALSE);
+		}
 	/* FIXME? else if (emb->playing_nth_item == emb->playlist_num_items) ? */
 	} else if (emb->num_items == 1) {
 		if (g_str_has_prefix (emb->current_uri, "file://") != FALSE) {
@@ -1322,19 +1503,24 @@ on_eos_event (GtkWidget *bvw, TotemEmbedded *emb)
 				bacon_video_widget_seek (emb->bvw, 0.0, NULL);
 			} else {
 				bacon_video_widget_close (emb->bvw);
-				totem_embedded_open_internal (emb, NULL /* FIXME? */);
+				totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 			}
 		} else {
 			bacon_video_widget_close (emb->bvw);
-			totem_embedded_open_internal (emb, NULL /* FIXME? */);
+			totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 		}
-		if (emb->repeat != FALSE && emb->autostart)
-			totem_embedded_play (emb, NULL);
 	} else if (emb->current) {
 		totem_embedded_open_playlist_item (emb, emb->current->next);
 	}
 }
 
+static gboolean
+skip_unplayable_stream (TotemEmbedded *emb)
+{
+	on_eos_event (BACON_VIDEO_WIDGET (emb->bvw), emb);
+	return FALSE;
+}
+
 static void
 on_error_event (BaconVideoWidget *bvw,
 		char *message,
@@ -1347,7 +1533,25 @@ on_error_event (BaconVideoWidget *bvw,
 		if (emb->is_browser_stream)
 			g_signal_emit (emb, signals[STOP_STREAM], 0);
 	
-		totem_embedded_set_state (emb, STATE_STOPPED);
+		totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
+
+		/* If we have a playlist, and that the current item
+		 * is < 60 seconds long, just go through it
+		 *
+		 * Same thing for all the items in a non-repeat playlist,
+		 * other than the last one
+		 *
+		 * FIXME we should mark streams as not playable though
+		 * so we don't loop through unplayable streams... */
+		if (emb->num_items > 1 && emb->current) {
+			TotemPlItem *item = emb->current->data;
+
+			if ((item->duration > 0 && item->duration < 60)
+			    || (!emb->repeat && emb->current->next)) {
+				g_idle_add ((GSourceFunc) skip_unplayable_stream, emb);
+				return;
+			}
+		}
 	}
 
 	if (fatal) {
@@ -1368,11 +1572,11 @@ static void
 on_tick (GtkWidget *bvw,
 		gint64 current_time,
 		gint64 stream_length,
-		float current_position,
+		double current_position,
 		gboolean seekable,
 		TotemEmbedded *emb)
 {
-	if (emb->state != STATE_STOPPED) {
+	if (emb->state != TOTEM_STATE_STOPPED) {
 		gtk_widget_set_sensitive (emb->seek, seekable);
 		gtk_widget_set_sensitive (emb->fs->seek, seekable);
 		if (emb->seeking == FALSE)
@@ -1391,10 +1595,20 @@ on_tick (GtkWidget *bvw,
 			(TOTEM_TIME_LABEL (emb->fs->time_label),
 			 current_time, stream_length);
 	}
+
+	g_signal_emit (emb, signals[TICK], 0,
+		       (guint32) current_time, (guint32) stream_length, totem_states[emb->state]);
 }
 
 static void
-property_notify_cb_volume (BaconVideoWidget *bvw, GParamSpec *spec,
+on_buffering (BaconVideoWidget *bvw, guint percentage, TotemEmbedded *emb)
+{
+//	g_message ("Buffering: %d %%", percentage);
+}
+
+static void
+property_notify_cb_volume (BaconVideoWidget *bvw,
+			   GParamSpec *spec,
 			   TotemEmbedded *emb)
 {
 	double volume;
@@ -1403,6 +1617,7 @@ property_notify_cb_volume (BaconVideoWidget *bvw, GParamSpec *spec,
 	
 	g_signal_handlers_block_by_func (emb->volume, cb_vol, emb);
 	gtk_scale_button_set_value (GTK_SCALE_BUTTON (emb->volume), volume);
+	totem_embedded_save_volume (emb, volume);
 	g_signal_handlers_unblock_by_func (emb->volume, cb_vol, emb);
 }
 
@@ -1515,6 +1730,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 			  int height)
 {
 	GtkWidget *child, *container, *image;
+	GtkWidget *popup_button;
 	BvwUseType type;
 	GError *err = NULL;
 	GConfClient *gc;
@@ -1559,7 +1775,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 	{
 		/* FIXME! */
 		/* FIXME construct and show error message */
-		totem_embedded_error_and_exit (_("The Totem plugin could not startup."), err != NULL ? err->message : _("No reason."), emb);
+		totem_embedded_error_and_exit (_("The Totem plugin could not be started."), err != NULL ? err->message : _("No reason."), emb);
 
 		if (err != NULL)
 			g_error_free (err);
@@ -1569,6 +1785,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 	emb->fs_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_widget_realize (emb->fs_window);
 	gtk_window_set_title (GTK_WINDOW (emb->fs_window), _("Totem Movie Player"));
+	gtk_window_set_default_icon_name ("totem");
 
 	emb->fs = totem_fullscreen_new (GTK_WINDOW (emb->fs_window));
 	totem_fullscreen_set_video_widget (emb->fs, emb->bvw);
@@ -1605,6 +1822,8 @@ totem_embedded_construct (TotemEmbedded *emb,
 			G_CALLBACK (on_video_button_press_event), emb);
 	g_signal_connect (G_OBJECT(emb->bvw), "tick",
 			G_CALLBACK (on_tick), emb);
+	g_signal_connect (G_OBJECT (emb->bvw), "buffering",
+			  G_CALLBACK (on_buffering), emb);
 
 	g_signal_connect (G_OBJECT (emb->bvw), "notify::volume",
 			  G_CALLBACK (property_notify_cb_volume), emb);
@@ -1638,6 +1857,12 @@ totem_embedded_construct (TotemEmbedded *emb,
 	g_signal_connect (G_OBJECT (emb->pp_button), "clicked",
 			  G_CALLBACK (on_play_pause), emb);
 
+	popup_button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "popup_button"));
+	g_signal_connect (G_OBJECT (popup_button), "toggled",
+			  G_CALLBACK (on_popup_button_toggled), emb);
+	g_signal_connect (G_OBJECT (popup_button), "button-press-event",
+			  G_CALLBACK (on_popup_button_button_pressed), emb);
+
 	gc = gconf_client_get_default ();
 	volume = ((double) gconf_client_get_int (gc, GCONF_PREFIX"/volume", NULL)) / 100.0;
 	g_object_unref (G_OBJECT (gc));
@@ -1655,7 +1880,10 @@ totem_embedded_construct (TotemEmbedded *emb,
 	emb->statusbar = TOTEM_STATUSBAR (gtk_builder_get_object (emb->xml, "statusbar"));
 	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (emb->statusbar), FALSE);
 
-	gtk_widget_set_size_request (emb->window, width, height);
+	if (!emb->hidden) {
+		gtk_widget_set_size_request (emb->window, width, height);
+		emb->scrsaver = totem_scrsaver_new ();
+	}
 
 #ifdef GNOME_ENABLE_DEBUG
 	child = GTK_WIDGET (gtk_builder_get_object (emb->xml, "controls"));
@@ -1688,7 +1916,13 @@ totem_embedded_construct (TotemEmbedded *emb,
 		g_object_unref (rcstyle);
 	}
 
-	totem_embedded_set_state (emb, STATE_STOPPED);
+	/* Create the cursor before setting the state */
+	if (!emb->hidden) {
+		emb->cursor = gdk_cursor_new_for_display
+			(gtk_widget_get_display (emb->window),
+			 GDK_HAND2);
+	}
+	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 
 	if (!emb->hidden) {
 		gtk_widget_show (emb->window);
@@ -1699,17 +1933,17 @@ totem_embedded_construct (TotemEmbedded *emb,
 					     GTK_WINDOW (emb->window), emb);
 	g_assert (emb->menuxml);
 
-	/* Create cursor and set the logo */
-	if (!emb->hidden) {
-		emb->cursor = gdk_cursor_new_for_display
-			(gtk_widget_get_display (emb->window),
-			 GDK_HAND2);
+	/* Set the logo and the button glow */
+	if (!emb->hidden && emb->autostart == FALSE) {
+		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
+		if (gtk_widget_get_direction (GTK_WIDGET (emb->bvw)) == GTK_TEXT_DIR_LTR)
+			totem_embedded_set_logo_by_name (emb, "gtk-media-play-ltr");
+		else
+			totem_embedded_set_logo_by_name (emb, "gtk-media-play-rtl");
+	} else {
 		totem_embedded_set_logo_by_name (emb, "totem");
 	}
 
-	if (!emb->hidden && emb->autostart == FALSE)
-		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
-
 	return TRUE;
 }
 
@@ -1800,10 +2034,21 @@ totem_embedded_unset_window (TotemEmbedded *embedded,
 	return TRUE;
 }
 
+static void
+totem_pl_item_free (gpointer data, gpointer user_data)
+{
+	TotemPlItem *item = (TotemPlItem *) data;
+
+	if (!item)
+		return;
+	g_free (item->uri);
+	g_free (item);
+}
+
 static void
 totem_embedded_clear_playlist (TotemEmbedded *embedded)
 {
-	g_list_foreach (embedded->playlist, (GFunc) g_free, NULL);
+	g_list_foreach (embedded->playlist, (GFunc) totem_pl_item_free, NULL);
 	g_list_free (embedded->playlist);
 
 	embedded->playlist = NULL;
@@ -1828,6 +2073,7 @@ entry_parsed (TotemPlParser *parser,
 	     gpointer data)
 {
 	TotemEmbedded *emb = (TotemEmbedded *) data;
+	TotemPlItem *item;
 
 	if (g_str_has_prefix (uri, "file://") != FALSE
 	    && g_str_has_prefix (emb->base_uri, "file://") == FALSE) {
@@ -1838,9 +2084,12 @@ entry_parsed (TotemPlParser *parser,
 	g_print ("added URI '%s'\n", uri);
 	g_hash_table_foreach (metadata, (GHFunc) entry_metadata_foreach, NULL);
 
-	//FIXMEchpe
-	//FIXME need new struct to hold that
-	emb->playlist = g_list_prepend (emb->playlist, g_strdup (uri));
+	item = g_new0 (TotemPlItem, 1);
+	item->uri = g_strdup (uri);
+	item->duration = totem_pl_parser_parse_duration (g_hash_table_lookup (metadata, "duration"), FALSE);
+	item->starttime = totem_pl_parser_parse_duration (g_hash_table_lookup (metadata, "starttime"), FALSE);
+
+	emb->playlist = g_list_prepend (emb->playlist, item);
 }
 
 static gboolean
@@ -1926,7 +2175,8 @@ parse_plugin_type (const gchar *option_name,
 		"gmp",
 		"complex",
 		"narrowspace",
-		"mully"
+		"mully",
+		"cone"
 	};
 	TotemPluginType type;
 
@@ -2085,7 +2335,7 @@ int main (int argc, char **argv)
 
 	emb = g_object_new (TOTEM_TYPE_EMBEDDED, NULL);
 
-	emb->state = LAST_STATE;
+	emb->state = TOTEM_STATE_INVALID;
 	emb->width = -1;
 	emb->height = -1;
 	emb->controller_hidden = arg_no_controls;
