@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemNarrowSpacePlugin.cpp 4180 2007-04-01 23:37:46Z hadess $
+ * $Id: totemNarrowSpacePlugin.cpp 4693 2007-09-11 14:51:38Z hadess $
  */
 
 #include <mozilla-config.h>
@@ -47,13 +47,15 @@ static const nsCID kClassID =
   { 0x97, 0x64, 0xf5, 0xab, 0x50, 0xa4, 0x0c, 0x06 } };
 
 static const char kClassDescription[] = "totemNarrowSpacePlugin";
-static const char kPluginDescription[] = "QuickTime Plug-in 7.1.3";
+#define QUICKTIME_VERSION "7.2.0"
+static const char kPluginDescription[] = "QuickTime Plug-in "QUICKTIME_VERSION;
 
 static const totemPluginMimeEntry kMimeTypes[] = {
 	{ "video/quicktime", "mov", NULL },
 	{ "video/mp4", "mp4", NULL },
 	{ "image/x-macpaint", "pntg", NULL },
 	{ "image/x-quicktime", "pict, pict1, pict2", "image/x-pict" },
+	{ "video/x-m4v", "m4v", NULL },
 };
 
 void*
@@ -68,10 +70,9 @@ totemScriptablePlugin::operator new (size_t aSize) CPP_THROW_NEW
 }
 
 totemScriptablePlugin::totemScriptablePlugin (totemPlugin *aPlugin)
-  : mPlugin(aPlugin),
-    mRate(1.0),
-    mVolume(100),
-    mPluginState(eState_Waiting)
+  : mPluginState(eState_Waiting),
+    mPlugin(aPlugin),
+    mVolume(100)
 {
   D ("%s ctor [%p]", kClassDescription, (void*) this);
 }
@@ -194,9 +195,10 @@ totemScriptablePlugin::SetControllerVisible(PRBool enabled)
 NS_IMETHODIMP
 totemScriptablePlugin::GetDuration(PRUint32 *_retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
-  *_retval = 0;
+  NS_ENSURE_STATE (IsValid());
+  *_retval = mPlugin->mDuration;
   return NS_OK;
 }
 
@@ -416,8 +418,10 @@ totemScriptablePlugin::SetMatrix(const nsACString & matrix)
 NS_IMETHODIMP
 totemScriptablePlugin::GetMaxBytesLoaded(PRUint32 *_retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
+  NS_ENSURE_STATE (IsValid());
+  *_retval = mPlugin->mBytesStreamed;
   return NS_OK;
 }
 
@@ -482,9 +486,10 @@ totemScriptablePlugin::SetMovieName(const nsACString & name)
 NS_IMETHODIMP
 totemScriptablePlugin::GetMovieSize(PRUint32 *_retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
-  *_retval = 0;
+  NS_ENSURE_STATE (IsValid());
+  *_retval = mPlugin->mBytesLength;
   return NS_OK;
 }
 
@@ -588,8 +593,6 @@ totemScriptablePlugin::SetPlayEveryFrame(PRBool enabled)
 NS_IMETHODIMP
 totemScriptablePlugin::GetPluginStatus(nsACString & _retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
-
   static const char *kState[] = {
     "Complete",
     "Error:<%d>",
@@ -611,9 +614,7 @@ totemScriptablePlugin::GetPluginStatus(nsACString & _retval)
 NS_IMETHODIMP
 totemScriptablePlugin::GetPluginVersion(nsACString & _retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
-
-  _retval.Assign ("7.0"); /* FIXME */
+  _retval.Assign (QUICKTIME_VERSION);
   return NS_OK;
 }
 
@@ -661,9 +662,7 @@ totemScriptablePlugin::GetQuickTimeLanguage(nsACString & _retval)
 NS_IMETHODIMP
 totemScriptablePlugin::GetQuickTimeVersion(nsACString & _retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
-
-  _retval.Assign ("7.0"); /* FIXME */
+  _retval.Assign (QUICKTIME_VERSION);
   return NS_OK;
 }
 
@@ -671,9 +670,15 @@ totemScriptablePlugin::GetQuickTimeVersion(nsACString & _retval)
 NS_IMETHODIMP
 totemScriptablePlugin::GetRate(float *_retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
-  *_retval = mRate;
+  NS_ENSURE_STATE (IsValid());
+
+  if (mPlugin->mState == TOTEM_STATE_PLAYING) {
+    *_retval = 1.0;
+  } else {
+    *_retval = 0.0;
+  }
   return NS_OK;
 }
 
@@ -683,7 +688,7 @@ totemScriptablePlugin::SetRate(float rate)
 {
   TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
 
-  mRate = rate;
+  /* mRate = rate; FIXME */
   return NS_OK;
 }
 
@@ -805,7 +810,7 @@ totemScriptablePlugin::Stop ()
 
   NS_ENSURE_STATE (IsValid ());
 
-  return mPlugin->DoCommand (TOTEM_COMMAND_STOP);
+  return mPlugin->DoCommand (TOTEM_COMMAND_PAUSE);
 }
 
 /* AUTF8String GetTarget (); */
@@ -848,9 +853,9 @@ totemScriptablePlugin::SetTiltAngle(float angle)
 NS_IMETHODIMP
 totemScriptablePlugin::GetTime(PRUint32 *_retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
-  *_retval = 0;
+  *_retval = mPlugin->mTime;
   return NS_OK;
 }
 
@@ -867,9 +872,9 @@ totemScriptablePlugin::SetTime(PRUint32 time)
 NS_IMETHODIMP
 totemScriptablePlugin::GetTimeScale(PRUint32 *_retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
-  *_retval = 0;
+  *_retval = 1000;
   return NS_OK;
 }
 
