@@ -71,12 +71,15 @@ GtkType    totem_playlist_get_type (void);
 GtkWidget *totem_playlist_new      (void);
 
 /* The application is responsible for checking that the mrl is correct
- * Handles directories, m3u playlists, and shoutcast playlists
  * @display_name is if you have a preferred display string for the mrl,
  * NULL otherwise
  */
-gboolean totem_playlist_add_mrl  (TotemPlaylist *playlist, const char *mrl,
+gboolean totem_playlist_add_mrl  (TotemPlaylist *playlist,
+				  const char *mrl,
 				  const char *display_name);
+gboolean totem_playlist_add_mrl_with_cursor (TotemPlaylist *playlist,
+					     const char *mrl,
+					     const char *display_name);
 
 void totem_playlist_save_current_playlist (TotemPlaylist *playlist,
 					   const char *output);
@@ -86,8 +89,6 @@ void totem_playlist_save_current_playlist_ext (TotemPlaylist *playlist,
 /* totem_playlist_clear doesn't emit the current_removed signal, even if it does
  * because the caller should know what to do after it's done with clearing */
 gboolean   totem_playlist_clear (TotemPlaylist *playlist);
-void       totem_playlist_clear_with_prefix (TotemPlaylist *playlist,
-					     const char *prefix);
 void       totem_playlist_clear_with_gnome_vfs_volume (TotemPlaylist *playlist,
 						       GnomeVFSVolume *volume);
 char      *totem_playlist_get_current_mrl (TotemPlaylist *playlist);
@@ -120,6 +121,7 @@ void       totem_playlist_set_shuffle (TotemPlaylist *playlist,
 				       gboolean shuffle);
 
 gboolean   totem_playlist_set_playing (TotemPlaylist *playlist, TotemPlaylistStatus state);
+TotemPlaylistStatus totem_playlist_get_playing (TotemPlaylist *playlist);
 
 void       totem_playlist_set_at_start (TotemPlaylist *playlist);
 void       totem_playlist_set_at_end (TotemPlaylist *playlist);
